<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/04/20
 * Time: 12.02
 */

namespace FiloBlu\Rma\Handlers;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface;
use Psr\Log\LoggerInterface;
use FiloBlu\Rma\Helper\ChangeSizeOrder;
use Magento\Framework\App\RequestInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Rma\Model\Item as ItemModel;


class CreateChangeSizeOrderHandler
{
    /**
     * @var ChangeSizeAttributeHelper
     */
    private $changeSizeAttributeHelper;
    /**
     * @var RmaExtensionAttributesRepositoryInterface
     */
    private $rmaExtensionAttributesRepository;

    /**
     * @var array
     */
    protected $changSizeItems = [];
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var ChangeSizeOrder
     */
    private $changeSizeOrder;
    /**
     * @var RequestInterface
     */
    private $request;
    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var ItemModel
     */
    private $itemModel;


    /**
     * CreateChangeSizeOrderHandler constructor.
     * @param ChangeSizeAttributeHelper $changeSizeAttributeHelper
     * @param RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository
     * @param LoggerInterface $logger
     * @param ChangeSizeOrder $changeSizeOrder
     * @param RequestInterface $request
     * @param ProductRepositoryInterface $productRepository
     * @param StoreManagerInterface $storeManager
     * @param ItemModel $itemModel
     */
    public function __construct(
        ChangeSizeAttributeHelper $changeSizeAttributeHelper,
        RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository,
        LoggerInterface $logger,
        ChangeSizeOrder $changeSizeOrder,
        RequestInterface $request,
        ProductRepositoryInterface $productRepository,
        StoreManagerInterface $storeManager,
        ItemModel $itemModel){
        $this->changeSizeAttributeHelper = $changeSizeAttributeHelper;
        $this->rmaExtensionAttributesRepository = $rmaExtensionAttributesRepository;
        $this->logger = $logger;
        $this->changeSizeOrder = $changeSizeOrder;
        $this->request = $request;
        $this->productRepository = $productRepository;
        $this->storeManager = $storeManager;
        $this->itemModel = $itemModel;
    }


    /**
     * @param \Magento\Rma\Model\Rma $subject
     * @param $result
     * @return mixed
     */
    public function checkIfCreateChangeSizeOrder(\Magento\Rma\Model\Rma $subject, $result){
        try {

            if ($result && isset($result['items'])) {

                $rmaItems = $result['items'];
                $parentProduct = null;
                $websiteId = $this->storeManager->getStore($subject->getStoreId())->getWebsiteId();
                foreach ($rmaItems as $item) {
                    if ($item->getStatus() == \Magento\Rma\Model\Rma\Source\Status::STATE_PENDING
                        && $item->getResolution() == $this->changeSizeAttributeHelper->getChangeSizeAttributeOptionId() && $this->changeSizeAttributeHelper->isChangeSizeEnabled($websiteId) ) {

                        if (!$this->rmaExtensionAttributesRepository->getChangeSizeOrderId($item->getId())) {

                            // check if customer
                            if($item->getExtensionAttributes() !== null && $item->getExtensionAttributes()->getChangeSize()!= null){
                                $item->setChangeSize($item->getExtensionAttributes()->getChangeSize());
                            }
                            if ($item->getChangeSize() != "" ||(isset($item->getData()[ChangeSizeAttributeHelper::CHANGE_SIZE_RMA_ATTRIBUTE_CODE])
                                    && $item->getData()[ChangeSizeAttributeHelper::CHANGE_SIZE_RMA_ATTRIBUTE_CODE] != "")) {

                                $changeProductId = $item->getChangeSize();
                                /** @var \Magento\Rma\Model\Item $item */
                                $parentProduct = $item->getProductSku() ?? $this->itemModel->setEntityId($item->getEntityId())->getProductSku();
                                $this->changSizeItems = array(
                                    'product_id' => $changeProductId,
                                    'qty' => $item['qty_requested'],
                                    'parent_product_sku' => $parentProduct,
                                    'rma_entity_id' => $item->getEntityId(),
                                    'original_order_item_id' => $item->getOrderItemId()
                                );
                            }
                            $customEmail = $subject->getCustomerCustomEmail();
                            if (isset($customEmail) && $customEmail != "") {
                                $subject->getOrder()->setCustomerEmail($customEmail);
                            }
                            $isLastItem=true;
                            if(next($rmaItems) == true){
                                $isLastItem=false;
                            }
                            // create an order for each item requested
                            $this->createChangeSizeOrder($subject->getOrder(), $this->changSizeItems, $subject->getEntityId(),$isLastItem, $parentProduct);

                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            return $result;
        } catch (\Throwable $t) {
            $this->logger->critical($t->getMessage());
            return $result;
        }
        return $result;
    }


    /**
     * @param $order
     * @param array $items
     * @param $rmaId
     * @param bool $isLast
     * @param null $parentProduct
     */
    protected function createChangeSizeOrder($order, array $items, $rmaId,bool $isLast, $parentProduct = null)
    {

        $orderData = [
            'order' => $order,
            'items' => $items,
            'rma_id' => $rmaId,
            'parent_product_sku' => $parentProduct,
            'request' => $this->request->getParams(),
            'is_last' => $isLast
        ];

        if ($parentProduct !== null && is_numeric($items['product_id'])){
            $this->changeSizeOrder->createMageOrder($orderData);
        }
    }

}